import $ from 'jquery';
import Button from 'ep-form/js/Element/Button';
import QuantityInput from '../Form/QuantityInput';

/**
 * Osnovna klasa za add-to-cart komponentu.
 */
export default class Abstract
{
    constructor(el)
    {
        let $el = $(el),
            $form = $el.find('form');

        el.addToCart = this;
        this.$form = $form;
        this._action = $form.data('ajax-action');
        this._button = new Button($form.find('button[type="submit"]')[0]);
        this._initQuantity($form);
        this._productId = parseInt($form.data('product-id'));

        $(this._quantity).on('change', $.proxy(this.onQuantityChange, this));
        $form.on('submit', $.proxy(this.onFormSubmit, this));

        $(this._quantity).trigger('change');
    }

    /**
     * Inicijalizira unos količine.
     *
     * @private
     */
    _initQuantity($form)
    {
        let $input = $form.find('.quantity-input');

        if ($input.length > 0) {
            this._quantity = new QuantityInput($input[0]);
        }
    }

    /**
     * Podaci za slanje ajaxom.
     */
    get request()
    {
        let request = this._request;
        request.action = this._action;
        request.form_id = this.$form.attr('id');
        return request;
    }

    // noinspection JSMethodCanBeStatic
    /**
     * Podaci za slanje ajaxom.
     *
     * @private
     */
    get _request()
    {
        throw "Podklase moraju implementirati ovu metodu."
    }

    /**
     * Kada submitamo formu.
     *
     * @param e
     */
    onFormSubmit(e)
    {
        e.preventDefault();

        this._button.$el.removeClass('added').addClass('loading');
        let data = this.request;

        $(document.body).trigger('adding_to_cart', [this._button.$el, data]);

        $.post(woocommerce_params.ajax_url, data, $.proxy(function(response) {
            $(document.body).trigger('added_to_cart', [response.fragments, response.cart_hash]);
            this._button.$el.removeClass('loading').addClass('added');
            $(document.body).trigger('wc_cart_button_updated', [this._button.$el]);
        }, this));
    }

    /**
     * Kada se odabrana količina promijeni.
     */
    onQuantityChange()
    {
        this._button.enabled = this._quantity.valid && this._quantity.value > 0;
    }
}