import $ from 'jquery';
import Abstract from './Abstract';

export default class Variable extends Abstract
{
    constructor(el)
    {
        super(el);

        let $el = $(el),
            $form = $el.find('form');

        this.$reset = $form.find('.add-to-cart__attributes-reset');
        this.$variationId = $form.find('input[name="variation_id"]');
        this.$display = $form.find('.add-to-cart__variation-display');
        this._initAttributes($form);
        this._variations = $form.data('product-variations');

        this.$reset.on('click', $.proxy(this.onReset, this));
    }

    _initAttributes()
    {
        this._selects = [];

        $.each(this.$form.find('select'), $.proxy(function(i, select) {
            let $select = $(select);
            $select.on('change', $.proxy(this.onChange, this));
            this._selects.push($select);
        }, this));
    }

    get chosenAttributes()
    {
        let data   = {},
            count  = 0,
            chosen = 0;

        $.each(this._selects, function(i, $select) {
            let name = $select.data('attribute_name') || $select.attr('name'),
                value = $select.val();

            if (value.length > 0) {
                chosen ++;
            }

            count ++;
            data[name] = value;
        });

        return {
            'count'      : count,
            'chosenCount': chosen,
            'data'       : data
        };
    }

    get variation()
    {
        if (!this._variations) {
            return null;
        }

        let attributes = this.chosenAttributes,
            data = attributes.data;

        if (attributes.count !== attributes.chosenCount) {
            return null;
        }

        for (let i = 0; i < this._variations.length; i++) {
            let variation = this._variations[i],
                match = true;

            for (let name in data) {
                // noinspection JSUnfilteredForInLoop
                let val1 = variation.attributes[name],
                    val2 = data[name];

                if (val1 !== undefined && val2 !== undefined && val1.length !== 0 && val2.length !== 0 && val1 !== val2) {
                    match = false;
                    break;
                }
            }

            if (match) {
                return variation;
            }
        }

        return null;
    }

    get _request()
    {
        let data = {
            type: 'variable',
            products: {}
        };

        data.products[this.variationId] = this._quantity.value;
        return data;
    }

    get variationId()
    {
        return parseInt(this.$variationId.val());
    }

    set variationId(value)
    {
        this.$variationId.val(value).trigger('change');
    }

    onQuantityChange()
    {
        this._button.enabled = !!this.variation && this._quantity.valid;
    }

    updateVariation()
    {
        let attributes = this.chosenAttributes;

        if (attributes.chosenCount > 0) {
            this.$reset.addClass('in');
        } else {
            this.$reset.removeClass('in');
        }

        if (attributes.count === attributes.chosenCount) {
            let variation = this.variation;

            if (variation) {
                let template = wp.template('variation-template'),
                    html = template({
                        variation: variation
                    });

                this.variationId = parseInt(variation.variation_id);
                this.$display.html(html);
            } else {
                let html = wp.template('unavailable-variation-template')();
                this.variationId = 0;
                this.$display.html(html);
            }
        }

        this.onQuantityChange();
    }

    onChange()
    {
        this.variationId = 0;
        this.$form.find('.wc-no-matching-variations').remove();
        this.$form.find('.add-to-cart__messages').html('');

        this.updateVariation();
    }

    onReset(e)
    {
        e.preventDefault();

        $.each(this._selects, function(i, $select) {
            $select.val('');
        });

        this.$display.html('');
        this.updateButton();
    }
}