import $ from 'jquery';
require('jquery-mousewheel');

export default class QuantityInput
{
    constructor(el)
    {
        let $el = $(el);

        el.quantityInput = this;
        this.$decrement = $el.find('.quantity-input__button--minus');
        this.$input = $el.find('input');
        this.$increment = $el.find('.quantity-input__button--plus');

        this.$decrement.on('click', $.proxy(this.decrement, this));
        this.$input.on('focus', $.proxy(this.onFocus, this));
        this.$input.on('mousewheel', $.proxy(this.onMouseWheel, this));
        this.$input.on('change', $.proxy(this.onChange, this));
        this.$increment.on('click', $.proxy(this.increment, this));

        this.buttons();
    }

    get valid()
    {
        return this.minValue <= this.value && this.value <= this.maxValue;
    }

    get value()
    {
        return parseFloat(this.$input.val());
    }

    set value(value)
    {
        value = Math.max(this.minValue, value);
        value = Math.min(this.maxValue, value);

        this.$input.val(value);
        this.$input.trigger('change');

        this.buttons();
    }

    get minValue()
    {
        return Math.max(parseFloat(this.$input.attr('min')), 0);
    }

    get maxValue()
    {
        let max = parseFloat(this.$input.attr('max'));
        return isNaN(max) ? 9999 : max;
    }

    get step()
    {
        return parseFloat(this.$input.attr('step')) || 1;
    }

    buttons()
    {
        this.$decrement.prop('disabled', this.value === this.minValue);
        this.$increment.prop('disabled', this.value === this.maxValue);
    }

    onChange()
    {
        $(this).trigger('change');
    }

    onFocus()
    {
        this.$input.select();
    }

    onMouseWheel(e)
    {
        e.preventDefault();

        if (e.deltaY > 0) {
            this.increment();
        } else {
            this.decrement();
        }
    }

    increment()
    {
        this.value = this.value + this.step;
    }

    decrement()
    {
        this.value = this.value - this.step;
    }
}