<?php
namespace Ep\Woo;

use Zend\ServiceManager\AbstractFactory\ReflectionBasedAbstractFactory;
use Zend\ServiceManager\Factory\InvokableFactory;

/**
 * Class ConfigProvider
 *
 * @package Ep\Woo
 * @author Elektronika Perhoč <support@ep.hr>
 * @link https://bitbucket.org/elektronika-perhoc/ep-woocommerce
 */
class ConfigProvider
{
    /**
     * Vraća konfiguraciju.
     *
     * @return array
     */
    public function __invoke()
    {
        return [
            'dependencies' => [
                'factories' => [
                    View\TwigExtension::class => InvokableFactory::class,
                ],
            ],

            'assets' => [
                'paths' => [
                    __DIR__ . '/../dist',
                ],
            ],

            'wp_plugins' => [
                'aliases' => [
                    'woo.category.automenu'            => Plugin\Category\AutoMenu::class,
                    'woo.category.uncategorized'       => Plugin\Category\Uncategorized::class,
                    'woo.coupons.userregisteredbefore' => Plugin\Coupon\UserRegisteredBefore::class,
                    'woo.orders.customstatus'          => Plugin\Order\CustomStatus::class,
                    'woo.payment.paypal'              => Plugin\Payment\Paypal::class,
                    'woo.products.loginforprices'      => Plugin\Product\LoginForPrices::class,
                ],

                'factories' => [
                    Plugin\Category\AutoMenu::class           => InvokableFactory::class,
                    Plugin\Category\Uncategorized::class      => InvokableFactory::class,
                    Plugin\Coupon\UserRegisteredBefore::class => InvokableFactory::class,
                    Plugin\Order\CustomStatus::class          => InvokableFactory::class,
                    Plugin\Payment\Paypal::class              => ReflectionBasedAbstractFactory::class,
                    Plugin\Product\LoginForPrices::class      => InvokableFactory::class,
                ],
            ],

            'twig' => [
                'template_paths' => [
                    'woo' => [
                        dirname(__DIR__) . '/view',
                    ],
                ],

                'extensions' => [
                    View\TwigExtension::class,
                ],
            ],

            'views' => [
                'aliases' => [
                    'woo.addtocart.grouped'  => View\AddToCart\Grouped::class,
                    'woo.addtocart.simple'   => View\AddToCart\Simple::class,
                    'woo.addtocart.variable' => View\AddToCart\Variable::class,
                    'woo.price'              => View\Price::class,
                    'woo.quantityinput'      => View\QuantityInput::class,
                ],
            ],

            'wp_widgets' => [
                'aliases' => [
                    'woo.categories' => Widget\Categories::class,
                ],

                'factories' => [
                    Widget\Categories::class => InvokableFactory::class,
                ],
            ],

            'wp' => [
                'plugins' => [
                    'woo.category.uncategorized',
                    'woo.orders.customstatus',
                ],

                'widgets' => [
                    'woo.categories',
                ],

                'views' => [
                    'woo.addtocart.grouped',
                    'woo.addtocart.simple',
                    'woo.addtocart.variable',
//                    'woo.price',
                    'woo.quantityinput',
                ],
            ],
        ];
    }
}