<?php
namespace Ep\Woo\View\AddToCart;

use Ep\Woo\View\AbstractView;
use WC_Product_Variation;

/**
 * Class AbstractAddToCart
 *
 * @package Ep\Woo\View\AddToCart
 * @author Elektronika Perhoč <support@ep.hr>
 * @link https://bitbucket.org/elektronika-perhoc/ep-woocommerce
 */
abstract class AbstractAddToCart extends AbstractView
{
    const AJAX_ACTION = 'ep_add_to_cart';

    /** @var bool */
    static $ajaxAdded = false;

    /**
     * Dodaje akcije i filtere u WordPress.
     */
    public function init(): void
    {
        parent::init();

        if (false === self::$ajaxAdded && defined('DOING_AJAX')) {
            add_action(sprintf('wp_ajax_%s', static::AJAX_ACTION), [$this, 'action']);
            add_action(sprintf('wp_ajax_nopriv_%s', static::AJAX_ACTION), [$this, 'action']);
            self::$ajaxAdded = true;
        }
    }

    /**
     * @throws \Exception
     */
    public function action()
    {
        ob_start();

        $cart = WC()->cart;

        if (isset($_POST['products']) && is_array($_POST['products'])) {
            $added = [];

            foreach ($_POST['products'] as $productId => $quantity) {
                $quantity = intval($quantity);
                $productId = apply_filters('woocommerce_add_to_cart_product_id', absint($productId));
                $isValid = apply_filters('woocommerce_add_to_cart_validation', true, $productId, $quantity);
                $productStatus = get_post_status($productId);

                $product = wc_get_product($productId);

                $isOk = $product instanceof WC_Product_Variation
                    ? $cart->add_to_cart($product->get_parent_id(), $quantity, $productId)
                    : $cart->add_to_cart($productId, $quantity);

                if ($isValid && 'publish' === $productStatus && false !== $isOk) {
                    $added[$productId] = $quantity;
                    do_action('woocommerce_ajax_added_to_cart', $productId);
                }
            }

            if (count($added) > 0) {
                wc_add_to_cart_message($added, true);
            }
        }

        ob_start();
        woocommerce_mini_cart();
        $miniCart = ob_get_clean();

        ob_start();
        wc_print_notices();
        $notices = ob_get_clean();

        $data = array(
            'fragments' => apply_filters('woocommerce_add_to_cart_fragments', [
                'div.widget_shopping_cart_content' => '<div class="widget_shopping_cart_content">' . $miniCart . '</div>',
                '#' . $_POST['form_id'] . ' .add-to-cart__messages' => '<div class="add-to-cart__messages">' . $notices . '</div>',
            ]),
            'cart_hash' => apply_filters('woocommerce_add_to_cart_hash',
                $cart->get_cart_for_session() ? md5(json_encode($cart->get_cart_for_session())) : '',
                $cart->get_cart_for_session()
            ),
        );

        wp_send_json($data);
    }
}