<?php
namespace Ep\Woo\View\AddToCart;

use Ep\Woo\View\AbstractView;
use Ep\Woo\View\Model\AddToCart;
use WC_Product_Variable;

/**
 * Omogućava dodavanje varijabilnih proizvoda u košaricu.
 *
 * @package Ep\Woo\View\AddToCart
 * @author Elektronika Perhoč <support@ep.hr>
 * @link https://bitbucket.org/elektronika-perhoc/ep-woocommerce
 */
class Variable extends AbstractView
{
    const TEMPLATE = '@woo/single-product/add-to-cart/variable.html.twig';
    const TEMPLATE_ATTRIBUTE = '@woo/single-product/add-to-cart/variable-attribute.html.twig';

    static $footerAdded = false;

    protected $templateOverrides = [
        'single-product/add-to-cart/variable.php',
    ];

    public function init(): void
    {
        parent::init();

        if (!self::$footerAdded) {
            add_action('wp_footer', [$this, 'wp_footer']);
            self::$footerAdded = true;
        }
    }

    /**
     * @throws \Twig_Error_Loader
     * @throws \Twig_Error_Runtime
     * @throws \Twig_Error_Syntax
     */
    public function wp_footer()
    {
        echo $this->twig->render('@woo/single-product/add-to-cart/variation.html.twig', [
            'unavailable_message' => __('Sorry, this product is unavailable. Please choose a different combination.', 'woocommerce')
        ]);
    }

    /**
     * Renderira view.
     *
     * @return string
     *
     * @throws \Twig_Error_Loader
     * @throws \Twig_Error_Runtime
     * @throws \Twig_Error_Syntax
     */
    public function render(): string
    {
        /** @var WC_Product_Variable $product */
        global $product;

        if (null === $product) {
            return '';
        }

        return $this->twig->render(self::TEMPLATE, [
            'this' => new AddToCart($product),
            'attributes' => $this->renderAttributes($product),
        ]);
    }

    /**
     * Renderira atributes proizvoda.
     *
     * @param WC_Product_Variable $product
     *
     * @return string
     *
     * @throws \Twig_Error_Loader
     * @throws \Twig_Error_Runtime
     * @throws \Twig_Error_Syntax
     */
    protected function renderAttributes(WC_Product_Variable $product): string
    {
        $attributes = $product->get_variation_attributes();
        $attributeKeys = array_keys($attributes);
        $last = end($attributeKeys);
        $rendered = [];

        foreach ($attributes as $name => $values) {
            $rendered[] = $this->renderAttribute($product, $name, $values, $last === $name);
        }

        return implode("\n", $rendered);
    }

    /**
     * Renderira jedan atribut proizvoda.
     *
     * @param WC_Product_Variable $product
     * @param string $name
     * @param array $values
     * @param bool $isLast
     *
     * @return string
     *
     * @throws \Twig_Error_Loader
     * @throws \Twig_Error_Runtime
     * @throws \Twig_Error_Syntax
     */
    protected function renderAttribute(WC_Product_Variable $product, string $name, array $values, $isLast = false): string
    {
        $slug = sanitize_title($name);

        $selected = isset($_REQUEST['attribute_' . $slug])
            ? wc_clean(stripslashes(urldecode($_REQUEST['attribute_' . $slug])))
            : $product->get_variation_default_attribute($name);

        ob_start();
        wc_dropdown_variation_attribute_options([
            'attribute' => $name,
            'class' => 'form-control',
            'options' => $values,
            'product' => $product,
            'selected' => $selected,
        ]);
        $dropdown = ob_get_clean();

        return $this->twig->render(self::TEMPLATE_ATTRIBUTE, [
            'dropdown' => $dropdown,
            'is_last' => $isLast,
            'label' => wc_attribute_label($name),
            'reset_text' => __('Clear', 'woocommerce'),
            'slug' => $slug,
        ]);
    }
}